# -*- coding: utf-8 -*-
"""clox functions."""
import os
import sys
import time
from calendar import TextCalendar as GregorianCalendar
import random
import datetime
import jdatetime
import argparse
import pytz
from art import tprint
from .jcalendar import TextCalendar as JalaliCalendar
from .params import HORIZONTAL_TIME_24H_FORMATS, VERTICAL_TIME_24H_FORMATS
from .params import HORIZONTAL_TIME_12H_FORMATS, VERTICAL_TIME_12H_FORMATS
from .params import CLOX_VERSION, DATE_FORMAT
from .params import TIMEZONES_LIST, COUNTRIES_LIST
from .params import ADDITIONAL_INFO, EXIT_MESSAGE
from .params import FACES_MAP, FACES_LIST, CALENDARS_LIST, DATE_SYSTEMS_LIST
from .params import HORIZONTAL_FACES_LIST_EXAMPLE, VERTICAL_FACES_LIST_EXAMPLE
from .params import CLOX_OVERVIEW, CLOX_REPO


def clox_info():
    """
    Print clox details.

    :return: None
    """
    tprint("Clox")
    tprint("V:" + CLOX_VERSION)
    print(CLOX_OVERVIEW)
    print(CLOX_REPO)


def clear_screen():
    """
    Clear screen function.

    :return: None
    """
    if sys.platform == "win32":
        os.system('cls')
    else:
        os.system('clear')


def get_face(index):
    """
    Return face name.

    :param index: face index
    :type index: int
    :return: face name as str
    """
    if index == -1:
        index = random.choice(sorted(FACES_MAP))
    return FACES_MAP[index]


def show_faces_list(vertical=False):
    """
    Show faces list.

    :param vertical: vertical mode flag
    :type vertical: bool
    :return: None
    """
    mode = "Horizontal"
    example = HORIZONTAL_FACES_LIST_EXAMPLE
    if vertical:
        example = VERTICAL_FACES_LIST_EXAMPLE
        mode = "Vertical"
    print("Faces list ({mode}):\n".format(mode=mode))
    for i in sorted(FACES_MAP):
        print('Face {index}\n'.format(index=i))
        tprint(example, font=get_face(i))
        print('=' * 80)


def show_timezones_list(country=None):
    """
    Show timezones list.

    :param country: country iso3166 code
    :type country: str
    :return: None
    """
    timezones_list = TIMEZONES_LIST
    country_name = "All"
    if country is not None:
        timezones_list = list(map(lambda x: x.upper(), pytz.country_timezones(country)))
        country_name = pytz.country_names[country]
    try:
        print("Timezones list ({country_name}):\n".format(country_name=country_name))
    except Exception:
        print("Timezones list ({country_name}):\n".format(country_name=country.upper()))
    for index, timezone in enumerate(sorted(timezones_list), 1):
        print("{index}. {timezone}".format(index=index, timezone=timezone))


def show_countries_list():
    """
    Show countries list.

    :return: None
    """
    print("Countries list:\n")
    for index, country_code in enumerate(sorted(COUNTRIES_LIST), 1):
        country_name = pytz.country_names[country_code]
        try:
            print("{index}. {country_code} - {country_name}".format(index=index,
                                                                    country_code=country_code, country_name=country_name))
        except Exception:
            print("{index}. {country_code} - {country_name}".format(index=index,
                                                                    country_code=country_code, country_name=country_code))


def print_calendar(mode="month", timezone=None, country=None, v_shift=0, h_shift=0, date_system="gregorian"):
    """
    Print calendar.

    :param mode: calendar mode
    :type mode: str
    :param timezone: timezone
    :type timezone: str
    :param country: country iso3166 code
    :type country: str
    :param v_shift: vertical shift
    :type v_shift: int
    :param h_shift: horizontal shift
    :type h_shift: int
    :param date_system: date system
    :type date_system: str
    :return: None
    """
    datetime_lib = datetime
    calendar_obj = GregorianCalendar()
    if date_system == "jalali":
        datetime_lib = jdatetime
        calendar_obj = JalaliCalendar()
    tz = None
    timezone_str = "Local"
    if country is not None:
        timezone = pytz.country_timezones(country)[0].upper()
    if timezone is not None:
        timezone_str = timezone
        tz = pytz.timezone(timezone)
    v_shift = max(0, v_shift)
    h_shift = max(0, h_shift)
    datetime_now = datetime_lib.datetime.now(tz=tz)
    current_date = datetime_now.strftime(DATE_FORMAT)
    print('\n' * v_shift, end='')
    print(" " * h_shift, end='')
    print("Today: {date}".format(date=current_date))
    print(" " * h_shift, end='')
    print("Timezone: {timezone}\n".format(timezone=timezone_str))
    calendar_str = calendar_obj.formatmonth(datetime_now.year, datetime_now.month)
    if mode == "year":
        calendar_str = calendar_obj.formatyear(datetime_now.year)
    print("\n".join([" " * h_shift + x for x in calendar_str.split("\n")]))


def run_clock(
        timezone=None,
        country=None,
        v_shift=0,
        h_shift=0,
        face=1,
        no_blink=False,
        vertical=False,
        hide_date=False,
        hide_timezone=False,
        am_pm=False,
        date_system="gregorian"):
    """
    Run clock.

    :param timezone: timezone
    :type timezone: str
    :param country: country iso3166 code
    :type country: str
    :param v_shift: vertical shift
    :type v_shift: int
    :param h_shift: horizontal shift
    :type h_shift: int
    :param face: face index
    :type face: int
    :param no_blink: no-blink flag
    :type no_blink: bool
    :param vertical: vertical mode flag
    :type vertical: bool
    :param hide_date: hide date flag
    :type hide_date: bool
    :param hide_timezone: hide timezone flag
    :type hide_timezone: bool
    :param am_pm: AM/PM mode flag
    :type am_pm: bool
    :param date_system: date system
    :type date_system: str
    :return: None
    """
    datetime_lib = datetime
    if date_system == "jalali":
        datetime_lib = jdatetime
    format_index = 0
    time_formats = HORIZONTAL_TIME_12H_FORMATS if am_pm else HORIZONTAL_TIME_24H_FORMATS
    if vertical:
        time_formats = VERTICAL_TIME_12H_FORMATS if am_pm else VERTICAL_TIME_24H_FORMATS
    tz = None
    timezone_str = "Local"
    if country is not None:
        timezone = pytz.country_timezones(country)[0].upper()
    if timezone is not None:
        timezone_str = timezone
        tz = pytz.timezone(timezone)
    v_shift = max(0, v_shift)
    h_shift = max(0, h_shift)
    face = get_face(face)
    while True:
        clear_screen()
        print('\n' * v_shift, end='')
        print(" " * h_shift, end='')
        datetime_now = datetime_lib.datetime.now(tz=tz)
        current_time = datetime_now.strftime(time_formats[format_index])
        current_date = datetime_now.strftime(DATE_FORMAT)
        tprint(current_time, font=face, sep="\n" + " " * h_shift)
        if not hide_date:
            print(" " * h_shift, end='')
            print(current_date)
        if not hide_timezone:
            print(" " * h_shift, end='')
            print("Timezone: {timezone}".format(timezone=timezone_str))
        time.sleep(1)
        if not no_blink:
            format_index = int(not format_index)


def main():
    """
    CLI main function.

    :return: None
    """
    parser = argparse.ArgumentParser()
    parser.epilog = ADDITIONAL_INFO
    parser.add_argument('--timezone', help='timezone', type=str.upper, choices=TIMEZONES_LIST)
    parser.add_argument('--country', help='country (iso3166 code)', type=str.upper, choices=COUNTRIES_LIST)
    parser.add_argument('--v-shift', help='vertical shift', type=int, default=0)
    parser.add_argument('--h-shift', help='horizontal shift', type=int, default=0)
    parser.add_argument('--version', help='version', nargs="?", const=1)
    parser.add_argument('--info', help='info', nargs="?", const=1)
    parser.add_argument('--face', help='face', type=int, choices=FACES_LIST, default=1)
    parser.add_argument('--faces-list', help='faces list', nargs="?", const=1)
    parser.add_argument('--timezones-list', help='timezones list', nargs="?", const=1)
    parser.add_argument('--countries-list', help='countries list', nargs="?", const=1)
    parser.add_argument('--no-blink', help='disable blinking mode', nargs="?", const=1)
    parser.add_argument('--vertical', help='vertical mode', nargs="?", const=1)
    parser.add_argument('--hide-date', help='hide date', nargs="?", const=1)
    parser.add_argument('--hide-timezone', help='hide timezone', nargs="?", const=1)
    parser.add_argument('--am-pm', help='AM/PM mode', nargs="?", const=1)
    parser.add_argument('--calendar', help='calendar mode', type=str.lower, choices=CALENDARS_LIST)
    parser.add_argument(
        '--date-system',
        help='date system',
        type=str.lower,
        choices=DATE_SYSTEMS_LIST,
        default="gregorian")
    args = parser.parse_args()
    if args.version:
        print(CLOX_VERSION)
    elif args.info:
        clox_info()
    elif args.faces_list:
        show_faces_list(vertical=args.vertical)
    elif args.timezones_list:
        show_timezones_list(args.country)
    elif args.countries_list:
        show_countries_list()
    elif args.calendar:
        print_calendar(
            mode=args.calendar,
            timezone=args.timezone,
            country=args.country,
            h_shift=args.h_shift,
            v_shift=args.v_shift,
            date_system=args.date_system)
    else:
        try:
            run_clock(
                timezone=args.timezone,
                country=args.country,
                h_shift=args.h_shift,
                v_shift=args.v_shift,
                face=args.face,
                no_blink=args.no_blink,
                vertical=args.vertical,
                hide_date=args.hide_date,
                hide_timezone=args.hide_timezone,
                am_pm=args.am_pm,
                date_system=args.date_system)
        except (KeyboardInterrupt, EOFError):
            print(EXIT_MESSAGE)



# -*- coding: utf-8 -*-
"""clox jalali calendar."""
# Reference: https://github.com/IKermani/jcalendar
import datetime
from itertools import repeat
import jdatetime

# Exception raised for bad input (with string parameter for details)
error = ValueError


# Exceptions raised for bad input
class IllegalMonthError(ValueError):
    """Illegal month error."""

    def __init__(self, month):
        """Initiate."""
        self.month = month

    def __str__(self):
        """Return string representation."""
        return "bad month number %r; must be 1-12" % self.month


class IllegalWeekdayError(ValueError):
    """Illegal weekday error."""

    def __init__(self, weekday):
        """Initiate."""
        self.weekday = weekday

    def __str__(self):
        """Return string representation."""
        return "bad weekday number %r; must be 0 (Shanbe) to 6 (Jom'e)" % self.weekday


# Constants for months referenced later
January = 1
February = 2

Farvardin = 1
Esfand = 12

# Number of days per month (except for Esfand in leap years)
mdays = [0, 31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29]

# Full and abbreviated names of weekdays
day_name = jdatetime.date.j_weekdays_en
day_abbr = jdatetime.date.j_weekdays_short_en
# day_abbr = ['Sh', 'Ye', 'Do', 'Se', 'Ch', 'Pa', 'Jo']

# Full and abbreviated names of months (1-based arrays!!!)
month_name = [0] + jdatetime.date.j_months_en
month_abbr = [0] + jdatetime.date.j_months_short_en

# Constants for weekdays
(MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY, SUNDAY) = range(7)
(DOSHANBE, SESHANBE, CHAHARSHANBE, PANJSHANBE, JOME, SHANBE, YEKSHANBE) = range(7)


def isleap(year):
    """Return True for leap years, False for non-leap years."""
    return jdatetime.date(year, 1, 1).isleap()


def leapdays(y1, y2):
    """Return number of leap years in range [y1, y2)."""
    leapdays = 0

    for year in range(y1, y2):
        if isleap(year):
            leapdays += 1

    return leapdays


def weekday(year, month, day):
    """Return week-day (0-6 ~ Mon-Sun)."""
    if not datetime.MINYEAR <= year <= datetime.MAXYEAR:
        year = 2000 + year % 400
    return jdatetime.date(year, month, day).weekday()


def monthrange(year, month):
    """Return weekday (0-6 ~ Mon-Sun) and number of days (28-31) for year, month."""
    if not 1 <= month <= 12:
        raise IllegalMonthError(month)
    day1 = weekday(year, month, 1)
    ndays = mdays[month] + (month == Esfand and isleap(year))
    return day1, ndays


def _monthlen(year, month):
    """Return length of month."""
    return mdays[month] + (month == Esfand and isleap(year))


def _prevmonth(year, month):
    """Return previous month."""
    if month == 1:
        return year - 1, 12
    else:
        return year, month - 1


def _nextmonth(year, month):
    """Return next month."""
    if month == 12:
        return year + 1, 1
    else:
        return year, month + 1


class Calendar(object):
    """Base calendar class. This class doesn't do any formatting. It simply provides data to subclasses."""

    def __init__(self, firstweekday=0):
        """Initiate."""
        self.firstweekday = firstweekday  # 0 = Doshanbe, 6 = Yekshanbe

    def getfirstweekday(self):
        """Get first weekday."""
        return self._firstweekday % 7

    def setfirstweekday(self, firstweekday):
        """Set first weekday."""
        self._firstweekday = firstweekday

    firstweekday = property(getfirstweekday, setfirstweekday)

    def iterweekdays(self):
        """Return an iterator for one week of weekday numbers starting with the configured first one."""
        for i in range(self.firstweekday, self.firstweekday + 7):
            yield i % 7

    def itermonthdates(self, year, month):
        """Return an iterator for one month."""
        for y, m, d in self.itermonthdays3(year, month):
            yield jdatetime.date(y, m, d)

    def itermonthdays(self, year, month):
        """Like itermonthdates(), but will yield day numbers. For days outside the specified month the day number is 0."""
        day1, ndays = monthrange(year, month)
        days_before = (day1 - self.firstweekday) % 7
        yield from repeat(0, days_before)
        yield from range(1, ndays + 1)
        days_after = (self.firstweekday - day1 - ndays) % 7
        yield from repeat(0, days_after)

    def itermonthdays2(self, year, month):
        """Like itermonthdates(), but will yield (day number, weekday number) tuples. For days outside the specified month the day number is 0."""
        for i, d in enumerate(self.itermonthdays(year, month), self.firstweekday):
            yield d, i % 7

    def itermonthdays3(self, year, month):
        """Like itermonthdates(), but will yield (year, month, day) tuples.  Can be used for dates outside of datetime.date range."""
        day1, ndays = monthrange(year, month)
        days_before = (day1 - self.firstweekday) % 7
        days_after = (self.firstweekday - day1 - ndays) % 7
        y, m = _prevmonth(year, month)
        end = _monthlen(y, m) + 1
        for d in range(end - days_before, end):
            yield y, m, d
        for d in range(1, ndays + 1):
            yield year, month, d
        y, m = _nextmonth(year, month)
        for d in range(1, days_after + 1):
            yield y, m, d

    def itermonthdays4(self, year, month):
        """Like itermonthdates(), but will yield (year, month, day, day_of_week) tuples. Can be used for dates outside of datetime.date range."""
        for i, (y, m, d) in enumerate(self.itermonthdays3(year, month)):
            yield y, m, d, (self.firstweekday + i) % 7

    def monthdatescalendar(self, year, month):
        """Return a matrix (list of lists) representing a month's calendar.Each row represents a week; week entries are datetime.date values."""
        dates = list(self.itermonthdates(year, month))
        return [dates[i:i + 7] for i in range(0, len(dates), 7)]

    def monthdays2calendar(self, year, month):
        """Return a matrix representing a month's calendar."""
        days = list(self.itermonthdays2(year, month))
        return [days[i:i + 7] for i in range(0, len(days), 7)]

    def monthdayscalendar(self, year, month):
        """Return a matrix representing a month's calendar."""
        days = list(self.itermonthdays(year, month))
        return [days[i:i + 7] for i in range(0, len(days), 7)]

    def yeardatescalendar(self, year, width=3):
        """Return the data for the specified year ready for formatting."""
        months = [
            self.monthdatescalendar(year, i)
            for i in range(Farvardin, Farvardin + 12)
        ]
        return [months[i:i + width] for i in range(0, len(months), width)]

    def yeardays2calendar(self, year, width=3):
        """Return the data for the specified year ready for formatting."""
        months = [
            self.monthdays2calendar(year, i)
            for i in range(Farvardin, Farvardin + 12)
        ]
        return [months[i:i + width] for i in range(0, len(months), width)]

    def yeardayscalendar(self, year, width=3):
        """Return the data for the specified year ready for formatting."""
        months = [
            self.monthdayscalendar(year, i)
            for i in range(Farvardin, Farvardin + 12)
        ]
        return [months[i:i + width] for i in range(0, len(months), width)]


class TextCalendar(Calendar):
    """Subclass of Calendar that outputs a calendar as a simple plain text similar to the UNIX program cal."""

    def prweek(self, theweek, width):
        """Print a single week (no newline)."""
        print(self.formatweek(theweek, width), end='')

    def formatday(self, day, weekday, width):
        """Return a formatted day."""
        if day == 0:
            s = ''
        else:
            s = '%2i' % day  # right-align single-digit days
        return s.center(width)

    def formatweek(self, theweek, width):
        """Return a single week in a string (no newline)."""
        return ' '.join(self.formatday(d, wd, width) for (d, wd) in theweek)

    def formatweekday(self, day, width):
        """Return a formatted week day name."""
        if width >= 9:
            names = day_name
        else:
            names = day_abbr
        return names[day][:width].center(width)

    def formatweekheader(self, width):
        """Return a header for a week."""
        return ' '.join(self.formatweekday(i, width) for i in self.iterweekdays())

    def formatmonthname(self, theyear, themonth, width, withyear=True):
        """Return a formatted month name."""
        s = month_name[themonth]
        if withyear:
            s = "%s %r" % (s, theyear)
        return s.center(width)

    def prmonth(self, theyear, themonth, w=0, l=0):
        """Print a month's calendar."""
        print(self.formatmonth(theyear, themonth, w, l), end='')

    def formatmonth(self, theyear, themonth, w=0, l=0):
        """Return a month's calendar string (multi-line)."""
        w = max(2, w)
        l = max(1, l)
        s = self.formatmonthname(theyear, themonth, 7 * (w + 1) - 1)
        s = s.rstrip()
        s += '\n' * l
        s += self.formatweekheader(w).rstrip()
        s += '\n' * l
        for week in self.monthdays2calendar(theyear, themonth):
            s += self.formatweek(week, w).rstrip()
            s += '\n' * l
        return s

    def formatyear(self, theyear, w=2, l=1, c=6, m=3):
        """Return a year's calendar as a multi-line string."""
        w = max(2, w)
        l = max(1, l)
        c = max(2, c)
        colwidth = (w + 1) * 7 - 1
        v = []
        a = v.append
        a(repr(theyear).center(colwidth * m + c * (m - 1)).rstrip())
        a('\n' * l)
        header = self.formatweekheader(w)
        for (i, row) in enumerate(self.yeardays2calendar(theyear, m)):
            # months in this row
            months = range(m * i + 1, min(m * (i + 1) + 1, 13))
            a('\n' * l)
            names = (self.formatmonthname(theyear, k, colwidth, False)
                     for k in months)
            a(formatstring(names, colwidth, c).rstrip())
            a('\n' * l)
            headers = (header for k in months)
            a(formatstring(headers, colwidth, c).rstrip())
            a('\n' * l)
            # max number of weeks for this row
            height = max(len(cal) for cal in row)
            for j in range(height):
                weeks = []
                for cal in row:
                    if j >= len(cal):
                        weeks.append('')
                    else:
                        weeks.append(self.formatweek(cal[j], w))
                a(formatstring(weeks, colwidth, c).rstrip())
                a('\n' * l)
        return ''.join(v)

    def pryear(self, theyear, w=0, l=0, c=6, m=3):
        """Print a year's calendar."""
        print(self.formatyear(theyear, w, l, c, m), end='')


c = TextCalendar()

firstweekday = c.getfirstweekday


def setfirstweekday(firstweekday):
    """Set first weekday."""
    if not DOSHANBE <= firstweekday <= YEKSHANBE:
        raise IllegalWeekdayError(firstweekday)
    c.firstweekday = firstweekday


monthcalendar = c.monthdayscalendar
prweek = c.prweek
week = c.formatweek
weekheader = c.formatweekheader
prmonth = c.prmonth
month = c.formatmonth
calendar = c.formatyear
prcal = c.pryear

# Spacing of month columns for multi-column year calendar
_colwidth = 7 * 3 - 1  # Amount printed by prweek()
_spacing = 6  # Number of spaces between columns


def format(cols, colwidth=_colwidth, spacing=_spacing):
    """Print multi-column formatting for year calendars."""
    print(formatstring(cols, colwidth, spacing))


def formatstring(cols, colwidth=_colwidth, spacing=_spacing):
    """Return a string formatted from n strings, centered within n columns."""
    spacing *= ' '
    return spacing.join(c.center(colwidth) for c in cols)


EPOCH = 1970
_EPOCH_ORD = jdatetime.date(EPOCH, 1, 1).toordinal()


def timegm(tuple):
    """Unrelated but handy function to calculate Unix timestamp from GMT."""
    year, month, day, hour, minute, second = tuple[:6]
    days = jdatetime.date(year, month, 1).toordinal() - _EPOCH_ORD + day - 1
    hours = days * 24 + hour
    minutes = hours * 60 + minute
    seconds = minutes * 60 + second
    return seconds
